@@ -207,8 +207,70 @@ static void get_name_property(
 
   return;
 }
-#else
+#else /* I18N_MB */
 #define GET_NAME_PROPERTY(a, b, c, d) get_name_property(a, b, c)
+#ifdef COMPOUND_TEXT
+static void get_name_property(
+  Status (func)(Display *, Window, XTextProperty *), Window w, char **ret_name)
+{
+  XTextProperty text_prop;
+  int num;
+  char **list;
+
+  /* Note: we dump the name because we cannot keep the list in memory
+   * without a lot of changes */
+  if (func(dpy, w, &text_prop) != 0)
+  {
+    if (text_prop.value)
+    {
+      if (text_prop.encoding == XA_STRING)
+      {
+        /* STRING encoding, use this as it is */
+	CopyString(&*ret_name, (char *)text_prop.value);
+	XFree(text_prop.value);
+      }
+      else
+      {
+        /* not STRING encoding, try to convert */
+        if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) == Success
+            && num > 0 && *list)
+	{
+          /* XXX: does not consider the conversion is REALLY succeeded */
+          XFree(text_prop.value); /* return of XGetWM(Icon)Name() */
+	  CopyString(&*ret_name, *list);
+	  if (list)
+	    XFreeStringList(list);
+        }
+	else
+	{
+          if (list)
+            XFreeStringList(list);
+          XFree(text_prop.value); /* return of XGet(Icon)WMName() */
+	  if (func(dpy, w, &text_prop))
+	  {
+	    CopyString(&*ret_name, (char *)text_prop.value);
+	    XFree(text_prop.value);
+	  }
+	  else
+	  {
+	    *ret_name = NoName;
+	  }
+        }
+      }
+    }
+    else
+    {
+      *ret_name = NoName;
+    }
+  }
+  else
+  {
+    *ret_name = NoName;
+  }
+
+  return;
+}
+#else /* COMPOUND_TEXT */
 static void get_name_property(
   Status (func)(Display *, Window, XTextProperty *), Window w, char **ret_name)
 {
@@ -221,7 +283,8 @@ static void get_name_property(
 
   return;
 }
-#endif
+#endif /* COMPOUND_TEXT */
+#endif /* I18N_MB */
 
 void setup_window_name(FvwmWindow *tmp_win)
 {
@@ -250,6 +313,7 @@ void setup_class_and_resource(FvwmWindow *tmp_win)
 void setup_wm_hints(FvwmWindow *tmp_win)
 {
   tmp_win->wmhints = XGetWMHints(dpy, tmp_win->w);
+  set_focus_model(tmp_win);
 }
 
 static void destroy_window_font(FvwmWindow *tmp_win)
@@ -305,18 +369,15 @@ void setup_window_font(
     if (tmp_win->title_font.y < tmp_win->title_font.font->ascent)
       tmp_win->title_font.y = tmp_win->title_font.font->ascent;
     tmp_win->title_g.height = height;
-    tmp_win->corner_width =
-      height + tmp_win->boundary_width;
   }
   else
   {
     height = tmp_win->title_font.height;
     tmp_win->title_font.y = tmp_win->title_font.font->ascent;
-    tmp_win->corner_width =
-      height + EXTRA_TITLE_FONT_HEIGHT + tmp_win->boundary_width;
     tmp_win->title_g.height =
       tmp_win->title_font.height + EXTRA_TITLE_FONT_HEIGHT;
   }
+    tmp_win->corner_width = tmp_win->title_g.height + tmp_win->boundary_width;
   if (!HAS_TITLE(tmp_win))
   {
     tmp_win->title_g.height = 0;
@@ -901,12 +962,39 @@ void destroy_resize_handle_windows(
   }
 }
 
+static void resize_resize_handle_windows(FvwmWindow *tmp_win)
+{
+        int i;
+
+        tmp_win->corner_width =
+                tmp_win->title_g.height + tmp_win->boundary_width;
+
+        if (HAS_BORDER(tmp_win))
+        {
+                /* Just dump the windows any old place and let SetupFrame take
+                 * care of the mess */
+                for (i = 0; i < 4; i++)
+                {
+                        XResizeWindow(
+                                dpy, tmp_win->corners[i],
+                                tmp_win->corner_width, tmp_win->corner_width);
+                }
+        }
+
+        return;
+}
+
 void change_resize_handle_windows(FvwmWindow *tmp_win)
 {
   if (HAS_BORDER(tmp_win) && tmp_win->sides[0] == None)
     setup_resize_handle_windows(tmp_win);
   else if (!HAS_BORDER(tmp_win) && tmp_win->sides[0] != None)
     destroy_resize_handle_windows(tmp_win, False);
+  else if (tmp_win->corner_width !=
+           tmp_win->title_g.height + tmp_win->boundary_width)
+  {
+    resize_resize_handle_windows(tmp_win);
+  }
 }
 
 void setup_frame_stacking(FvwmWindow *tmp_win)
@@ -961,9 +1049,24 @@ void setup_frame_attributes(
   XSetWindowAttributes xswa;
 
   /* Backing_store is controlled on the client, decor_w, title & buttons */
-  xswa.backing_store = pstyle->flags.use_backing_store
-		       ? Scr.use_backing_store : NotUseful;
+  switch (pstyle->flags.use_backing_store)
+  {
+  case BACKINGSTORE_DEFAULT:
+	  xswa.backing_store = tmp_win->initial_backing_store;
+	  break;
+  case BACKINGSTORE_ON:
+	  xswa.backing_store = Scr.use_backing_store;
+	  break;
+  case BACKINGSTORE_OFF:
+  default:
+	  xswa.backing_store = NotUseful;
+	  break;
+  }
   XChangeWindowAttributes(dpy, tmp_win->w, CWBackingStore, &xswa);
+  if (pstyle->flags.use_backing_store == BACKINGSTORE_OFF)
+  {
+    xswa.backing_store = NotUseful;
+  }
   XChangeWindowAttributes(dpy, tmp_win->decor_w, CWBackingStore, &xswa);
   if (HAS_TITLE(tmp_win))
   {
@@ -1309,6 +1412,9 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   setup_class_and_resource(tmp_win);
   setup_wm_hints(tmp_win);
 
+  /* save a copy of the backing store attribute */
+  tmp_win->initial_backing_store = tmp_win->attr.backing_store;
+
   /****** basic style and decor ******/
   /* If the window is in the NoTitle list, or is a transient, dont decorate it.
    * If its a transient, and DecorateTransients was specified, decorate anyway.
@@ -1551,6 +1657,15 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
       tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y,
       tmp_win->frame_g.width, tmp_win->frame_g.height, False);
   }
+  if (!IS_SHADED(tmp_win) && !IS_ICONIFIED(tmp_win))
+  {
+    /* TK always wants some special treatment: If the window is simply mapped,
+     * the tk menus come up at funny Y coordinates.  Tell it its geometry
+     * *again* to work around this problem. */
+    SendConfigureNotify(
+      tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y, tmp_win->frame_g.width,
+      tmp_win->frame_g.height, 0, False);
+  }
   if (!XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY, &JunkWidth,
 		    &JunkHeight, &JunkBW,  &JunkDepth))
   {
@@ -1589,7 +1704,10 @@ void FetchWmProtocols (FvwmWindow *tmp)
     for (i = 0, ap = protocols; i < n; i++, ap++)
     {
       if (*ap == (Atom)_XA_WM_TAKE_FOCUS)
+      {
 	SET_WM_TAKES_FOCUS(tmp, 1);
+	set_focus_model(tmp);
+      }
       if (*ap == (Atom)_XA_WM_DELETE_WINDOW)
 	SET_WM_DELETES_WINDOW(tmp, 1);
     }
@@ -1608,7 +1726,10 @@ void FetchWmProtocols (FvwmWindow *tmp)
       for (i = 0, ap = protocols; i < nitems; i++, ap++)
       {
 	if (*ap == (Atom)_XA_WM_TAKE_FOCUS)
+	{
 	  SET_WM_TAKES_FOCUS(tmp, 1);
+	  set_focus_model(tmp);
+	}
 	if (*ap == (Atom)_XA_WM_DELETE_WINDOW)
 	  SET_WM_DELETES_WINDOW(tmp, 1);
       }
@@ -1915,7 +2036,11 @@ static void free_text_property(char **ptext, char ***ptext_list)
 #define FREE_TEXT_PROPERTY(a, b) free_text_property(a)
 static void free_text_property(char **ptext)
 {
+#ifdef COMPOUND_TEXT
+  free(*ptext);
+#else
   XFree(*ptext);
+#endif
   *ptext = NULL;
 }
 #endif
@@ -2032,6 +2157,7 @@ void destroy_window(FvwmWindow *tmp_win)
   /* already done above? */
   if (!IS_SCHEDULED_FOR_DESTROY(tmp_win))
   {
+    SET_SCHEDULED_FOR_DESTROY(tmp_win, 1);
     /****** adjust fvwm internal windows and the focus ******/
 
     adjust_fvwm_internal_windows(tmp_win);
@@ -2140,6 +2266,7 @@ void RestoreWithdrawnLocation(
   XWindowChanges xwc;
   rectangle naked_g;
   rectangle unshaded_g;
+  XSetWindowAttributes xswa;
 
   if(!tmp)
     return;
@@ -2199,6 +2326,11 @@ void RestoreWithdrawnLocation(
       }
     }
   }
+
+  /* restore initial backing store setting on window */
+  xswa.backing_store = tmp->initial_backing_store;
+  XChangeWindowAttributes(dpy, tmp->w, CWBackingStore, &xswa);
+  /* reparent to root window */
   XReparentWindow(
     dpy, tmp->w, (parent == None) ? Scr.Root : parent, xwc.x, xwc.y);
 
@@ -2331,10 +2463,16 @@ void CaptureOneWindow(
     destroy_window(fw);
     Event.xmaprequest.window = w;
     HandleMapRequestKeepRaised(keep_on_top_win, fw);
-    if (!fFvwmInStartup)
+    /* HandleMapRequestKeepRaised may have destroyed the fw if the window
+     * vanished while in AddWindow(), so don't access fw anymore before
+     * checking if it is a valid window. */
+    if (check_if_fvwm_window_exists(fw))
     {
-      SET_MAP_PENDING(fw, 0);
-      SET_MAPPED(fw, is_mapped);
+      if (!fFvwmInStartup)
+      {
+	SET_MAP_PENDING(fw, 0);
+	SET_MAPPED(fw, is_mapped);
+      }
     }
     /* Clean out isIconicState here, otherwise all new windos may start
      * iconified. */
