@@ -597,7 +597,7 @@ void CMD_ResizeMove(F_CMD_ARGS)
 
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_RESIZE, ButtonPress))
     return;
-  if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+  if (tmp_win == NULL)
     return;
   if (IS_FIXED(tmp_win))
     return;
@@ -694,9 +694,14 @@ static void InteractiveMove(
 
   if (do_start_at_pointer)
   {
-    XQueryPointer(
+    if (XQueryPointer(
       dpy, Scr.Root, &JunkRoot, &JunkChild, &DragX, &DragY, &JunkX, &JunkY,
-      &JunkMask);
+      &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      DragX = 0;
+      DragY = 0;
+    }
   }
   else
   {
@@ -835,12 +840,19 @@ static void AnimatedMoveAnyWindow(
     XMoveWindow(dpy,w,currentX,currentY);
     if (fWarpPointerToo == True)
     {
-      XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		    &JunkX,&JunkY,&pointerX,&pointerY,&JunkMask);
-      pointerX += currentX - lastX;
-      pointerY += currentY - lastY;
-      XWarpPointer(dpy,None,Scr.Root,0,0,0,0,
-		   pointerX,pointerY);
+      if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+			&JunkX,&JunkY,&pointerX,&pointerY,&JunkMask) == False)
+      {
+	/* pointer is on a different screen */
+	pointerX = currentX;
+	pointerY = currentY;
+      }
+      else
+      {
+	pointerX += currentX - lastX;
+	pointerY += currentY - lastY;
+      }
+      XWarpPointer(dpy,None,Scr.Root,0,0,0,0, pointerX,pointerY);
     }
     if (tmp_win && !IS_SHADED(tmp_win))
     {
@@ -944,6 +956,11 @@ static void move_window_doit(F_CMD_ARGS, Bool do_animate, int mode)
 
   if (IS_FIXED(tmp_win))
     return;
+  if (check_if_function_allowed(F_MOVE, tmp_win, True, NULL) == 0)
+  {
+    return;
+  }
+
 
   /* gotta have a window */
   w = tmp_win->frame;
@@ -1028,6 +1045,10 @@ static void move_window_doit(F_CMD_ARGS, Bool do_animate, int mode)
 
     if (n != 2 || fPointer)
       InteractiveMove(&w, tmp_win, &FinalX, &FinalY, eventp, fPointer);
+    else if (IS_ICONIFIED(tmp_win))
+    {
+      SET_ICON_MOVED(tmp_win, 1);
+    }
   }
 
   if (w == tmp_win->frame)
@@ -1562,11 +1583,27 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
   /* prevent flicker when paging */
   SET_WINDOW_BEING_MOVED_OPAQUE(tmp_win, do_move_opaque);
 
-  XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,&xl, &yt,
-                &JunkX, &JunkY, &button_mask);
+  {
+    int xl_bak;
+    int yt_bak;
+
+    xl_bak = xl;
+    yt_bak = yt;
+    if (XQueryPointer(
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &xl, &yt, &JunkX, &JunkY,
+	  &button_mask) == False)
+    {
+      /* pointer is on a different screen */
+      xl = xl_bak;
+      yt = yt_bak;
+    }
+    else
+    {
+      xl += XOffset;
+      yt += YOffset;
+    }
+  }
   button_mask &= DEFAULT_ALL_BUTTONS_MASK;
-  xl += XOffset;
-  yt += YOffset;
   xl_orig = xl;
   yt_orig = yt;
 
@@ -1761,6 +1798,10 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
       break;
 
     case MotionNotify:
+      if (Event.xmotion.same_screen == False)
+      {
+	continue;
+      }
       xl = Event.xmotion.x_root;
       yt = Event.xmotion.y_root;
       if (xl > 0 && xl < Scr.MyDisplayWidth - 1)
@@ -2212,7 +2253,7 @@ void CMD_Resize(F_CMD_ARGS)
   bad_window = False;
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_RESIZE, ButtonPress))
     return;
-  if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+  if (tmp_win == NULL)
     return;
 
   ResizeWindow = tmp_win->frame;
@@ -2603,6 +2644,10 @@ void CMD_Resize(F_CMD_ARGS)
       break;
 
     case MotionNotify:
+      if (Event.xmotion.same_screen == False)
+      {
+	continue;
+      }
       if (!fForceRedraw)
       {
 	x = Event.xmotion.x_root;
@@ -3176,7 +3221,7 @@ void CMD_Maximize(F_CMD_ARGS)
 
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_SELECT,ButtonRelease))
     return;
-  if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+  if (tmp_win == NULL)
     return;
 
   if (check_if_function_allowed(F_MAXIMIZE,tmp_win,True,NULL) == 0)
